tic

%parameters
m = 0.45; %mortality
a = 0.95; %average water availability

v = 10; %advection parameter -> steepness of hill
cy=2; %channe depth


%spatial grid, simulation parameters
tmax = 1000;
dT=0.05;
Lx =200;
Ly =50;
Nx = 512;
Ny = 128;

%pick tplt=integer times tout
tout= 1; %time between output of solution measures
tplt= 10*tout; %time between solution outputs


x = Lx*(1:Nx)'/Nx;
y = Ly*(1:Ny)'/Ny;
[xx, yy]= meshgrid(x,y);
dx=x(2)-x(1);
dy=y(2)-y(1);
dA=dx*dy;

kx = 2*pi*[0:Nx/2-1 0 -Nx/2+1:-1]'/Lx; % wave numbers
ky = 2*pi*[0:Ny/2-1 0 -Ny/2+1:-1]'/Ly; % wave numbers
[kkx, kky]=meshgrid(kx,ky);


k0=2*pi/Ly;
%elevation function, not including constant slope in x, given by v
%hx=0;
%hxx=0;
hy=-v*cy*k0*sin(k0*yy);
hyy=-v*cy*k0^2*cos(k0*yy);


%initial condition
rng(462)
noise=rand(Ny,Nx,2);
B = (a/2/m+real(sqrt((a/2/m)^2-1)))*(0.975+0.05*noise(:,:,1));
Bhat = fft2(B);
B=real(ifft2(Bhat));
W = m*(a/2/m-real(sqrt((a/2/m)^2-1)))*(0.975+0.05*noise(:,:,2));
What=fft2(W);
W=real(ifft2(W));


%Define equation
KlausLB = -(m + kkx.^2+kky.^2); % Fourier multipliers
%KlausLW = (-1 + hxx + hyy  +  1i*hx.*kkx + 1i*hy.*kky); % Fourier multipliers
KlausLW = (-1 + 1i*v*kkx);

[Ew, E2w, Qw,f1w,f2w,f3w] = etdrk4coeffs(KlausLW,dT);
[Eb, E2b, Qb,f1b,f2b,f3b] = etdrk4coeffs(KlausLB,dT);


% Main time-stepping loop:
nmax = round(tmax/dT); nplt = floor(tplt/dT); nout=floor(tout/dT);
Nplt=floor(nmax/nplt)+1;
tts = zeros(Nplt,1);
Nout=floor(nmax/nout)+1;
Btop=zeros(Nout,1); Bmid=zeros(Nout,1); Bbot=zeros(Nout,1);
Bnorm=zeros(Nout,1);
ttm=zeros(Nout,1);


nnt=1;npl=1;
ttm(nnt)=0;
Bnorm(nnt)=sqrt(sum(sum(B.^2))*dA);
Bbot(nnt)=B(Ny/2,Nx/2);
Bmid(nnt)=B(Ny/4,Nx/2);
Btop(nnt)=B(Ny,Nx/2);


figure(1);


xfilter=(floor(Nx/4)+1):(ceil(3*Nx/4)+1);
yfilter=(floor(Ny/4)+1):(ceil(3*Ny/4)+1);

for n = 1:nmax
    t = n*dT;
    
    %%%%%%%%%%%%%
    B0hat=Bhat;
    W0hat=What;
    B0hat(yfilter,:)=0;
    B0hat(:,xfilter)=0;
    W0hat(yfilter,:)=0;
    W0hat(:,xfilter)=0;
    
    %step 1
    %%%%%%%%%%%%%%
    %NonlinB
    NLb0 = fft2(W.*B.^2) ;
    NLb0(yfilter,:)=0;
    NLb0(:,xfilter)=0;
    %NonlinW
    dyW=ifft2(1i*kky.*W0hat);
    NLw0 = fft2( a  +  hy.*dyW  +  hyy.*W ) - NLb0;
    NLw0(yfilter,:)=0;
    NLw0(:,xfilter)=0;
    %a
    B1hat = E2b.*B0hat + Qb.*NLb0;
    W1hat = E2w.*W0hat + Qw.*NLw0;
    
    
    %step 2
    %%%%%%%%%%%%%%
    %NonlinB
    Wt=real(ifft2(W1hat));
    Bt=real(ifft2(B1hat));
    NLb1 = fft2(Wt.*Bt.^2);
    NLb1(yfilter,:)=0;
    NLb1(:,xfilter)=0;
    %NonlinW
    dyW=real(ifft2(1i*kky.*W1hat));
    NLw1 = fft2( a  +  hy.*dyW  +  hyy.*Wt ) - NLb1;
    NLw1(yfilter,:)=0;
    NLw1(:,xfilter)=0;
    %b
    B2hat = E2b.*B0hat + Qb.*NLb1;
    W2hat = E2w.*W0hat + Qw.*NLw1;
    
    
    %step 3
    %%%%%%%%%%%%%%
    %NonlinB
    Wt=real(ifft2(W2hat));
    Bt=real(ifft2(B2hat));
    NLb2 = fft2(Wt.*Bt.^2);
    NLb2(yfilter,:)=0;
    NLb2(:,xfilter)=0;
    %NonlinW
    dyW=real(ifft2(1i*kky.*W2hat));
    NLw2 = fft2( a  +  hy.*dyW  +  hyy.*Wt ) - NLb2;
    NLw2(yfilter,:)=0;
    NLw2(:,xfilter)=0;
    %c
    B3hat = E2b.*B1hat + Qb.*(2*NLb2-NLb0);
    W3hat = E2w.*W1hat + Qw.*(2*NLw2-NLw0);
    
    %Final output
    %%%%%%%%%%%%%%
    %NonlinB
    Wt=real(ifft2(W3hat));
    Bt=real(ifft2(B3hat));
    NLb3 = fft2(Wt.*Bt.^2);
    NLb3(yfilter,:)=0;
    NLb3(:,xfilter)=0;
    %NonlinW
    dyW=real(ifft2(1i*kky.*W3hat));
    NLw3 = fft2( a  +  hy.*dyW  +  hyy.*Wt ) - NLb3;
    NLw3(yfilter,:)=0;
    NLw3(:,xfilter)=0;
    %B,W
    Bhat = Eb.*B0hat + f1b.*NLb0 + 2*f2b.*(NLb1+NLb2) + f3b.*NLb3;
    What = Ew.*W0hat + f1w.*NLw0 + 2*f2w.*(NLw1+NLw2) + f3w.*NLw3;
    
    B=real(ifft2(Bhat));
    W=real(ifft2(What));
    
    
    %save data
    if mod(n,nout)==0
        nnt=nnt+1;
        
        ttm(nnt)=t;
        Bnorm(nnt)=sqrt(sum(sum(B.^2))*dA);
        Bbot(nnt)=B(Ny/2,Nx/2);
        Bmid(nnt)=B(Ny/4,Nx/2);
        Btop(nnt)=B(Ny,Nx/2);
        
        %plot simulation output to screen
        if mod(n,nplt)==0
            npl=npl+1;
            
            surf(x,y,B,'EdgeColor','none', 'facecolor', 'interp'),
            box on, axis equal tight, caxis([0,a/m]),
            colormap(flipud(summer)), colorbar;
            view(2); xlabel('x'); ylabel('y');
     
            %drawnow
            print(['Bt' num2str(t) '.png'], '-dpng')
            tts(npl) = t;
            disp(['t=' num2str(t)]);
        end
    end
end

xout=[ttm,Bnorm,Btop,Bmid,Bbot];

% Plot results:
figure(3);
surf(x,y,B,'EdgeColor','none', 'facecolor', 'interp'),
box on, axis equal tight, caxis([0,a/m]),
colormap(flipud(summer)), colorbar;
view(2); xlabel('x'); ylabel('y');
%print(['Bt' num2str(t) '.eps'], '-depsc2')

figure(4);
surf(x,y,W,'EdgeColor','none', 'facecolor', 'interp'),
box on, axis equal tight, caxis([0,a]),
colormap(flipud(parula)), colorbar;
view(2); xlabel('x'); ylabel('y');
%print(['Wt' num2str(t) '.eps'], '-depsc2')


%save output to files.
save('Bout.dat','xout','-ascii');
save('Bsol.dat','B','-ascii');
save('Wsol.dat','W','-ascii');
toc
